@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,8 @@
 
 */
 
-#ifndef ESCSEQ_H
-#define ESCSEQ_H
+#ifndef UIM_FEP_ESCSEQ_H
+#define UIM_FEP_ESCSEQ_H
 
 struct point_tag {
   int row;
