@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -32,7 +32,7 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 #include <stdio.h>
 #ifdef HAVE_SYS_TYPES_H
@@ -60,43 +60,74 @@
 #include <sys/stat.h>
 #endif
 
+#include <uim/uim.h>
+#include <uim/uim-helper.h>
+#include "uim-posix.h"
 #include "udsock.h"
 
 static int s_send_sockfd = -1;
 static int s_recv_sockfd = -1;
 static struct sockaddr_un s_servaddr;
 
+uim_bool
+get_ud_path(char *path, int len)
+{
+  if (len <= 0)
+    return UIM_FALSE;
+
+  if (!uim_get_config_path(path, len, !uim_helper_is_setugid())) {
+    path[0] = '\0';
+    return UIM_FALSE;
+  }
+
+  if (strlcat(path, "/fep", len) >= (size_t)len) {
+    path[0] = '\0';
+    return UIM_FALSE;
+  }
+
+  if (!uim_check_dir(path)) {
+    path[0] = '\0';
+    return UIM_FALSE;
+  }
+
+  return UIM_TRUE;
+}
+
+
 const char *usersockname(const char *file)
 {
   static char buf[UNIX_PATH_MAX];
   char filebuf[UNIX_PATH_MAX];
+  char sock_dir[UNIX_PATH_MAX];
+
   if (file != NULL && file[0] == '/') {
     return file;
   }
+
   if (file == NULL) {
-    struct passwd *pw = getpwuid(getuid());
-    /* snprintf'\0'դƤ */
-    snprintf(filebuf, UNIX_PATH_MAX, "uim-fep-%s", pw->pw_name);
+    strlcpy(filebuf, "backtick", UNIX_PATH_MAX);
   } else {
-    /* strncpy'\0'դʤ */
-    strncpy(filebuf, file, UNIX_PATH_MAX - 1);
-    filebuf[UNIX_PATH_MAX - 1] = '\0';
+    strlcpy(filebuf, file, UNIX_PATH_MAX);
   }
-  if (getenv("TMP")) {
-    snprintf(buf, UNIX_PATH_MAX, "%s/%s", getenv("TMP"), filebuf);
-  } else {
-    snprintf(buf, UNIX_PATH_MAX, "/tmp/%s", filebuf);
+
+  if (!get_ud_path(sock_dir, sizeof(sock_dir))) {
+    sendline("uim-fep cannot create directory");
+    exit(EXIT_FAILURE);
   }
+  snprintf(buf, UNIX_PATH_MAX, "%s/%s", sock_dir, filebuf);
+
   return buf;
 }
 
 void init_sendsocket(const char *sock_path)
 {
-  sock_path = usersockname(sock_path);
+  const char *path;
+
+  path = usersockname(sock_path);
   s_send_sockfd = socket(PF_UNIX, SOCK_DGRAM, 0);
   memset(&s_servaddr, 0, sizeof(s_servaddr));
   s_servaddr.sun_family = AF_UNIX;
-  strncpy(s_servaddr.sun_path, sock_path, UNIX_PATH_MAX - 1);
+  strlcpy(s_servaddr.sun_path, path, UNIX_PATH_MAX);
 }
 
 void sendline(const char *buf)
@@ -110,17 +141,19 @@ void sendline(const char *buf)
  */
 void init_recvsocket(const char *sock_path)
 {
-  sock_path = usersockname(sock_path);
-  unlink(sock_path);
+  const char *path;
+
+  path = usersockname(sock_path);
+  unlink(path);
   s_recv_sockfd = socket(PF_UNIX, SOCK_DGRAM, 0);
   memset(&s_servaddr, 0, sizeof(s_servaddr));
   s_servaddr.sun_family = AF_UNIX;
-  strncpy(s_servaddr.sun_path, sock_path, UNIX_PATH_MAX - 1);
+  strlcpy(s_servaddr.sun_path, path, UNIX_PATH_MAX);
   if (bind(s_recv_sockfd, (struct sockaddr *)&s_servaddr, sizeof(s_servaddr)) < 0) {
-    perror(sock_path);
+    perror(path);
     exit(EXIT_FAILURE);
   }
-  chmod(sock_path, S_IRUSR|S_IWUSR);
+  chmod(path, S_IRUSR|S_IWUSR);
 }
 
 void close_socket(void)
