@@ -1,6 +1,6 @@
 ;;; im.scm: Core IM management functions for uim
 ;;;
-;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -38,7 +38,6 @@
 (require "load-action.scm")
 
 ;; config
-(define enable-im-switch #f)
 (define default-im-name #f)
 
 ;; preedit attributes: should be moved to another file
@@ -246,18 +245,28 @@
 
 (define next-im
   (lambda (name)
-    (let* ((im-names (map car im-list))
+    (let* ((im-names enabled-im-list)
 	   (im-rest (memq name im-names)))
       (or (and im-rest
 	       (pair? (cdr im-rest))
 	       (cadr im-rest))
 	  (car im-names)))))
 
+(define next-im-for-switch-im
+  (lambda (name)
+    (let ((im (next-im name)))
+      (or
+       (and
+	switch-im-skip-direct-im?
+	(eq? im 'direct)
+	(next-im im))
+       im))))
+
 ;; 'switch-im' is not a API but an IM-switching method. Don't confuse with
 ;; im-switch-im
 (define switch-im
   (lambda (uc name)
-    (im-switch-im uc (next-im name))))
+    (im-switch-im uc (next-im-for-switch-im name))))
 
 ;; FIXME: Input states are kept only if the state is appeared in the
 ;; toolbar.
@@ -444,7 +453,7 @@
        ((and enable-im-toggle?
 	     (toggle-im-key? key state))
 	(toggle-im uc c))
-       ((and enable-im-switch
+       ((and enable-im-switch?
 	     (switch-im-key? key state))
 	(switch-im uc (im-name im)))
        ((modifier-key? key state)
