@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -152,6 +152,7 @@
     ((("g" "y" "z"). ())(("" "" "") ("" "" "") ("" "" "")))
     ((("g" "z"). ())(("" "" "") ("" "" "")))
     ((("h" "d"). ())(("" "" "") ("" "" "")))
+    ((("h" "f"). ())("" "" ""))
     ((("h" "g" "a"). ())(("" "" "") ("" "" "")))
     ((("h" "g" "d"). ())(("" "" "") ("" "" "") ("" "" "")))
     ((("h" "g" "e"). ())(("" "" "") ("" "" "")))
@@ -241,6 +242,7 @@
     ((("l" "i"). ())("" "" ""))
     ((("l" "o"). ())("" "" ""))
     ((("l" "u"). ())("" "" ""))
+    ((("l" "w" "a"). ())("" "" ""))
     ((("l" "y" "a"). ())("" "" ""))
     ((("l" "y" "e"). ())(("" "" "") ("" "" "")))
     ((("l" "y" "i"). ())(("" "" "") ("" "" "")))
@@ -284,6 +286,7 @@
     ((("n"). ())("" "" ""))
     ((("n" "b"). ())(("" "" "") ("" "" "ʎ")))
     ((("n" "d"). ())(("" "" "") ("" "" "")))
+    ((("n" "f"). ())("" "" ""))
     ((("n" "g" "a"). ())(("" "" "") ("" "" "")))
     ((("n" "g" "d"). ())(("" "" "") ("" "" "") ("" "" "")))
     ((("n" "g" "e"). ())(("" "" "") ("" "" "")))
@@ -317,6 +320,7 @@
     ((("n" "y" "z"). ())(("" "" "") ("" "" "") ("" "" "")))
     ((("n" "z"). ())(("" "" "") ("" "" "")))
     ((("p" "d"). ())(("" "" "͎") ("" "" "")))
+    ((("p" "f"). ())(("" "" "Ύ") ("" "" "")))
     ((("p" "g" "a"). ())(("" "" "ˎ") ("" "" "")))
     ((("p" "g" "d"). ())(("" "" "ˎ") ("" "" "") ("" "" "")))
     ((("p" "g" "e"). ())(("" "" "ˎ") ("" "" "")))
@@ -436,6 +440,7 @@
     ((("t" "y" "z"). ())(("" "" "") ("" "" "") ("" "" "")))
     ((("t" "z"). ())(("" "" "") ("" "" "")))
     ((("w" "d"). ())(("" "" "") ("" "" "") ("" "" "")))
+    ((("w" "f"). ())(("" "" "") ("" "" "")))
     ((("w" "k"). ())(("" "" "") ("" "" "") ("" "" "")))
     ((("w" "l"). ())(("" "" "") ("" "" "") ("" "" "")))
     ((("w" "n"). ())(("" "" "") ("" "" "")))
@@ -472,6 +477,7 @@
     ((("x" "w"). ())(("" "" "") ("" "" "") ("" "" "")))
     ((("x" "z"). ())(("" "" "") ("" "" "") ("" "" "")))
     ((("y" "e"). ())(("" "" "") ("" "" "")))
+    ((("y" "f"). ())("" "" ""))
     ((("y" "h"). ())(("" "" "") ("" "" "")))
     ((("y" "i"). ())("" "" ""))
     ((("y" "j"). ())(("" "" "") ("" "" "")))
