@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -15,10 +15,10 @@
 ;;;    may be used to endorse or promote products derived from this software
 ;;;    without specific prior written permission.
 ;;;
-;;; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+;;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND
 ;;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
 ;;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 ;;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 ;;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -32,11 +32,16 @@
 ;;
 (define py-init-handler
   (lambda (id im arg)
-    (require "PY.scm")
+    (require "py.scm")
     (generic-context-new id im py-rule #f)))
 
 (generic-register-im
- 'py "zh_CN" "GB18030" (N_ "PinYin input method") py-init-handler)
+ 'py
+ "zh_CN"
+ "UTF-8"
+ (N_ "New Pinyin (Simplified)")
+ (N_ "Pinyin input method (Simplified Chinese version)")
+ py-init-handler)
 
 (define pyunihan-init-handler
   (lambda (id im arg)
@@ -44,7 +49,12 @@
     (generic-context-new id im pyunihan-rule #f)))
 
 (generic-register-im
- 'pyunihan "zh_CN" "UTF-8" (N_ "PinYin input method (Unicode version)") pyunihan-init-handler)
+ 'pyunihan
+ "zh"
+ "UTF-8"
+ (N_ "Pinyin (Unicode)")
+ (N_ "Pinyin input method (Unicode version)")
+ pyunihan-init-handler)
 
 (define pinyin-big5-init-handler
   (lambda (id im arg)
@@ -52,4 +62,9 @@
     (generic-context-new id im pinyin-big5-rule #f)))
 
 (generic-register-im
- 'pinyin-big5 "zh_TW:zh_HK" "UTF-8" (N_ "PinYin input method (Big5 version)") pinyin-big5-init-handler)
+ 'pinyin-big5
+ "zh_TW:zh_HK"
+ "UTF-8"
+ (N_ "Pinyin (Traditional)")
+ (N_ "Pinyin input method (Traditional Chinese version)")
+ pinyin-big5-init-handler)
