@@ -2,7 +2,7 @@
 ;;  About    : unit tests for memq, memv, member
 ;;
 ;;  Copyright (C) 2006 YAMAMOTO Kengo <yamaken AT bp.iij4u.or.jp>
-;;  Copyright (c) 2007 SigScheme Project <uim AT freedesktop.org>
+;;  Copyright (c) 2007-2008 SigScheme Project <uim-en AT googlegroups.com>
 ;;
 ;;  All rights reserved.
 ;;
@@ -31,14 +31,14 @@
 ;;  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-(load "./test/unittest.scm")
+(require-extension (unittest))
 
 (define tn test-name)
 
-(define elm0 (lambda () #f))
-(define elm1 (lambda () #f))
-(define elm2 (lambda () #f))
-(define elm3 (lambda () #f))
+(define elm0 (lambda () 0))
+(define elm1 (lambda () 1))
+(define elm2 (lambda () 2))
+(define elm3 (lambda () 3))
 (define nil  '())
 (define cdr3 (cons elm3 nil))
 (define cdr2 (cons elm2 cdr3))
