@@ -1,6 +1,6 @@
 #!/usr/bin/env gosh
 
-;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -1095,7 +1095,7 @@
 		 (uim '(string-escape "\a\B")))
    ;; complex
    (assert-equal "\"\\\"a string\\\" in two-line\\nstring\\n\""
-		 (uim '(string-escape "\"a\ string\" in two-line\nstring\n"))))
+		 (uim '(string-escape "\"a string\" in two-line\nstring\n"))))
 
   ("test compose"
    (uim '(define test-list '(0 1 2 3 4 5)))
