@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,8 +34,8 @@
 //
 // dispatcher of Xlib events for window.
 //
-#ifndef _xdispatch_h_included_
-#define _xdispatch_h_included_
+#ifndef UIM_XIM_XDISPATCH_H
+#define UIM_XIM_XDISPATCH_H
 
 #include <X11/X.h>
 
