@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,8 @@
 */
 
 // -*- C++ -*-
-#ifndef _ximserver_h_included_
-#define _ximserver_h_included_
+#ifndef UIM_XIM_XIMSERVER_H
+#define UIM_XIM_XIMSERVER_H
 
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -44,6 +44,7 @@
 #include "uim/uim.h"
 #include "compose.h"
 
+#define UIM_XIM_USE_NEW_PAGE_HANDLING 1
 
 // preedit ornament
 #define PE_NORMAL 0
@@ -53,6 +54,9 @@
 
 typedef wchar_t uchar;
 typedef std::list<uchar> uString;
+#if UIM_XIM_USE_NEW_PAGE_HANDLING
+typedef std::vector<const char *> CandList;
+#endif
 struct pe_ustring {
     uString s;
     int stat;
@@ -169,6 +173,10 @@ public:
     void candidate_shift_page(int direction);
     void candidate_deactivate();
     void candidate_update();
+#if UIM_XIM_USE_NEW_PAGE_HANDLING
+    void prepare_page_candidates(int page);
+    int prepare_page_candidates_by_index(int index);
+#endif
     void update_prop_list(const char *str);
     void update_prop_label(const char *str);
     bool hasActiveCandwin();
@@ -181,6 +189,7 @@ public:
     void configuration_changed();
     void switch_app_global_im(const char *name);
     void switch_system_global_im(const char *name);
+    void set_need_hilite_selected_cand(bool set);
 public:
     static void commit_cb(void *ptr, const char *str);
     static void clear_cb(void *ptr);
@@ -210,10 +219,17 @@ private:
     uim_context mUc;
     bool mCandwinActive;
     int mDisplayLimit;
+#if UIM_XIM_USE_NEW_PAGE_HANDLING
+    int mNumCandidates;
+#endif
     int mNumPage;
     int current_cand_selection;
     int current_page;
+    bool need_hilite_selected_cand;
     std::vector<const char *> active_candidates;
+#if UIM_XIM_USE_NEW_PAGE_HANDLING
+    std::vector<CandList> mCandidateSlot;
+#endif
     char *mEngineName;
     char *mLocaleName;
     bool mCaretStateShown;
